/**
 * Tests if the standard state transition works correctly.
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_1

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>

namespace sta
{
	namespace tasty
	{
		class DummyThread : public tacos::TacosThread
		{
		private:
			/* data */
		public:
			DummyThread()
				: tacos::TacosThread{"Dummy", osPriorityNormal}
			{}

			void func() override 
			{
				STA_TASTY_INIT();

				STA_TASTY_ASSERT(tacos::getState() == 0);

				// Transitioning from 0 to 1 should work correctly.
				tacos::requestState(0, 1);
				STA_TASTY_ASSERT(tacos::getState() == 1);

				// Transitioning from 0 to 5 should fail because we are in state 1.
				tacos::requestState(0, 5);
				STA_TASTY_ASSERT(tacos::getState() == 1);

				// Transitioning from 7 to 0 should fail aswell
				tacos::requestState(7, 0);
				STA_TASTY_ASSERT(tacos::getState() == 1);

				// Transitioning from 4 to 5 should also fail.
				tacos::requestState(5, 4);
				STA_TASTY_ASSERT(tacos::getState() == 1);

				STA_TASTY_TERMINATE();
			}
		};

		void onTastyInit()
		{
			tacos::addThread<DummyThread>({0, 1, 2, 3, 4, 5, 6, 7});
		}
	} // namespace tasty
} // namespace sta

#endif // TASTY_CASE_1
