/**
 * Test lockout timers in combination with timed transitions.
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_4

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>

namespace sta
{
	namespace tasty
	{
		class DummyThread : public tacos::TacosThread
		{
		private:
			/* data */
		public:
			DummyThread()
				: tacos::TacosThread{"Dummy", osPriorityNormal}
			{}

			void func() override 
			{
				STA_TASTY_INIT();

				STA_TASTY_ASSERT(tacos::getState() == 0);

                // Trigger a state transition with lockout of 100 ticks.
                tacos::requestState(0, 1, 100);

				// Schedule a state transition to be executed after 50 ticks.
				tacos::setStateTimed(1, 2, 50);

				// This should have no effect.
				STA_TASTY_ASSERT(tacos::getState() == 1);

				// Wait for 60 ticks.
				this->sleep(60);
				
                // The state transition should have been blocked by the lockout timer
                STA_TASTY_ASSERT(tacos::getState() == 1);

				// Schedule a state transition to be executed in 41 ticks.
				tacos::setStateTimed(1, 2, 41);

				this->sleep(50);

				// This state transition should get through.
				STA_TASTY_ASSERT(tacos::getState() == 2);

                STA_TASTY_TERMINATE();
			}
		};

		void onTastyInit()
		{
			tacos::addThread<DummyThread>({0, 1, 2, 3, 4, 5, 6, 7});
		}
	} // namespace tasty
} // namespace sta

#endif // TASTY_CASE_4
