/*
 * case8.cpp
 *
 *  Created on: Jan 24, 2024
 *      Author: Dario
 */

/**
 * Tests if threads are properly terminated and started because of state transitions. The thread status is updated accordingly and no memory leaks occur.
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_9

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>
#include <sta/rtos/debug/heap_stats.hpp>

namespace sta
{
    namespace tasty
    {
    	class ThreadA : public tacos::TacosThread
		{
		public:
			ThreadA()
				: tacos::TacosThread{"A", osPriorityNormal}
			{

			}

			void func() override
			{
				osThreadYield();
			}
		};

		class ThreadB : public tacos::TacosThread
		{
		public:
			ThreadB()
				: tacos::TacosThread{"B", osPriorityNormal}
			{

			}

			void func() override
			{
				osThreadYield();
			}
		};


		std::shared_ptr<ThreadA> thread_A;
		std::shared_ptr<ThreadB> thread_B;


    	class Observer : public tacos::TacosThread
		{
		public:
			Observer()
				: tacos::TacosThread{"Observer", osPriorityNormal}
			{

			}

			void func() override
			{
				size_t mallocs = sta::rtos::getNumAllocs();

				STA_TASTY_ASSERT(thread_A->isRunning());
				STA_TASTY_ASSERT(!thread_B->isRunning());
				STA_TASTY_ASSERT(thread_B->getStatus() == tacos::ThreadStatus::STOPPED);

				this->sleep(20);

				tacos::requestState(0, 1);

				this->sleep(5);

				STA_TASTY_ASSERT(!thread_A->isRunning());
				STA_TASTY_ASSERT(thread_B->isRunning());
				STA_TASTY_ASSERT(thread_A->getStatus() == tacos::ThreadStatus::STOPPED);
				STA_TASTY_ASSERT(sta::rtos::getNumAllocs() == mallocs + 2);

				tacos::requestState(1, 0);

				this->sleep(5);

				STA_TASTY_ASSERT(thread_A->isRunning());
				STA_TASTY_ASSERT(!thread_B->isRunning());
				STA_TASTY_ASSERT(thread_B->getStatus() == tacos::ThreadStatus::STOPPED);
				STA_TASTY_ASSERT(sta::rtos::getNumAllocs() == mallocs + 2);

				tacos::requestState(0, 1);

				STA_TASTY_ASSERT(!thread_A->isRunning());
				STA_TASTY_ASSERT(thread_B->isRunning());
				STA_TASTY_ASSERT(thread_A->getStatus() == tacos::ThreadStatus::STOPPED);
				STA_TASTY_ASSERT(sta::rtos::getNumAllocs() == mallocs + 2);

				STA_TASTY_TERMINATE();
			}
		};

        void onTastyInit()
        {
        	STA_TASTY_INIT();

            thread_A = tacos::addThread<ThreadA>({0});
            thread_B = tacos::addThread<ThreadB>({1});
            tacos::addThread<Observer>({0, 1});
        }
    } // namespace tasty
}

#endif // TASTY_CASE_9
